'use strict';

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _Promise = require('babel-runtime/core-js/promise')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;

var _path2 = require('path');

var _path3 = _interopRequireDefault(_path2);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _requireHacker = require('require-hacker');

var _requireHacker2 = _interopRequireDefault(_requireHacker);

var _toolsLog = require('./tools/log');

var _toolsLog2 = _interopRequireDefault(_toolsLog);

var _helpers = require('./helpers');

var _common = require('./common');

// using ES6 template strings
// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/template_strings

var webpack_isomorphic_tools = (function () {
	function webpack_isomorphic_tools(options) {
		_classCallCheck(this, webpack_isomorphic_tools);

		this.hooks = [];
		this.cached_assets = [];

		// take the passed in options
		this.options = _helpers.convert_from_camel_case(_helpers.clone(options));

		// add missing fields, etc
		_common.normalize_options(this.options);

		// if Webpack aliases are supplied, enable aliasing
		if (this.options.alias) {
			this.enable_aliasing();
		}

		// logging
		this.log = new _toolsLog2['default']('webpack-isomorphic-tools', { debug: this.options.debug });

		this.log.debug('instantiated webpack-isomorphic-tools with options', this.options);
	}

	// sets development mode flag to whatever was passed (or true if nothing was passed)
	// (development mode allows asset hot reloading when used with webpack-dev-server)

	webpack_isomorphic_tools.prototype.development = function development(flag) {
		// set development mode flag
		this.options.development = _helpers.exists(flag) ? flag : true;

		if (this.options.development) {
			this.log.debug('entering development mode');
		} else {
			this.log.debug('entering production mode');
		}

		// allows method chaining
		return this;
	};

	// returns a mapping to read file paths for all the user specified asset types
	// along with a couple of predefined ones: javascripts and styles

	webpack_isomorphic_tools.prototype.assets = function assets() {
		// when in development mode
		if (this.options.development) {
			// webpack and node.js start in parallel
			// so webpack-assets.json might not exist on the very first run
			// if a developer chose not to use the .server() method with a callback
			// (or if a developer chose not to wait for a Promise returned by the .server() method)
			if (!_fs2['default'].existsSync(this.webpack_assets_path)) {
				this.log.error('"' + this.webpack_assets_path + '" not found. Most likely it hasn\'t yet been generated by Webpack. Using an empty stub instead.');
				return _common.default_webpack_assets();
			}
		}

		return require(this.webpack_assets_path);
	};

	// clear the require.cache (only used in developer mode with webpack-dev-server)

	webpack_isomorphic_tools.prototype.refresh = function refresh() {
		// ensure this is development mode
		if (!this.options.development) {
			throw new Error('.refresh() called in production mode. Did you forget to call .development() method on your webpack-isomorphic-tools server instance?');
		}

		this.log.debug('flushing require() caches');

		// uncache webpack-assets.json file
		// this.log.debug(' flushing require() cache for webpack assets json file')
		// this.log.debug(` (was cached: ${typeof(require.cache[this.webpack_assets_path]) !== 'undefined'})`)
		delete require.cache[this.webpack_assets_path];

		// uncache cached assets
		for (var _iterator = this.cached_assets, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _getIterator(_iterator);;) {
			var _ref;

			if (_isArray) {
				if (_i >= _iterator.length) break;
				_ref = _iterator[_i++];
			} else {
				_i = _iterator.next();
				if (_i.done) break;
				_ref = _i.value;
			}

			var _path = _ref;

			this.log.debug(' flushing require() cache for ' + _path);
			delete require.cache[_path];
		}

		// no assets are cached now
		this.cached_assets = [];
	};

	// Makes `webpack-isomorphic-tools` aware of Webpack aliasing feature.
	// https://webpack.github.io/docs/resolving.html#aliasing
	// The `aliases` parameter corresponds to `resolve.alias`
	// in your Webpack configuration.
	// If this method is used it must be called before the `.server()` method.

	webpack_isomorphic_tools.prototype.enable_aliasing = function enable_aliasing() {
		var _this = this;

		// mount require() hook
		this.alias_hook = _requireHacker2['default'].global_hook('aliasing', function (path) {
			// alias the path
			var aliased_path = _common.alias(path, _this.options.alias);

			// if an alias is found, require() the correct path
			if (aliased_path) {
				var result = require(aliased_path);
				return _requireHacker2['default'].to_javascript_module_source(result);
			}
		}, { precede_node_loader: true });

		// allows method chaining
		return this;
	};

	// Initializes server-side instance of `webpack-isomorphic-tools`
	// with the base path for your project, then calls `.register()`,
	// and after that calls .wait_for_assets(callback).
	//
	// The `project_path` parameter must be identical
	// to the `context` parameter of your Webpack configuration
	// and is needed to locate `webpack-assets.json`
	//  which is output by Webpack process.
	//
	// sets up "project_path" option
	// (this option is required on the server to locate webpack-assets.json)

	webpack_isomorphic_tools.prototype.server = function server(project_path, callback) {
		var _this2 = this;

		// project base path, required to locate webpack-assets.json
		this.options.project_path = project_path;

		// resolve webpack-assets.json file path
		this.webpack_assets_path = _path3['default'].resolve(this.options.project_path, this.options.webpack_assets_file_path);

		// register require() hooks
		this.register();

		// when ready:

		// if callback is given, call it back
		if (callback) {
			// call back when ready
			return this.wait_for_assets(callback);
		}
		// otherwise resolve a Promise
		else {
			// no callback given, return a Promise
			return new _Promise(function (resolve, reject) {
				return _this2.wait_for_assets(resolve);
			});
		}
	};

	// Registers Node.js require() hooks for the assets
	//
	// This is what makes the `requre()` magic work on server.
	// These `require()` hooks must be set before you `require()`
	// any of your assets
	// (e.g. before you `require()` any React components
	// `require()`ing your assets).
	//
	// read this article if you don't know what a "require hook" is
	// http://bahmutov.calepin.co/hooking-into-node-loader-for-fun-and-profit.html

	webpack_isomorphic_tools.prototype.register = function register() {
		this.log.debug('registering require() hooks for assets');

		// for each user specified asset type,
		// register a require() hook for each file extension of this asset type
		for (var _iterator2 = _Object$keys(this.options.assets), _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _getIterator(_iterator2);;) {
			var _ref2;

			if (_isArray2) {
				if (_i2 >= _iterator2.length) break;
				_ref2 = _iterator2[_i2++];
			} else {
				_i2 = _iterator2.next();
				if (_i2.done) break;
				_ref2 = _i2.value;
			}

			var asset_type = _ref2;

			var description = this.options.assets[asset_type];

			for (var _iterator3 = description.extensions, _isArray3 = Array.isArray(_iterator3), _i3 = 0, _iterator3 = _isArray3 ? _iterator3 : _getIterator(_iterator3);;) {
				var _ref3;

				if (_isArray3) {
					if (_i3 >= _iterator3.length) break;
					_ref3 = _iterator3[_i3++];
				} else {
					_i3 = _iterator3.next();
					if (_i3.done) break;
					_ref3 = _i3.value;
				}

				var extension = _ref3;

				this.register_extension(extension, description);
			}
		}

		// allows method chaining
		return this;
	};

	// registers a require hook for a particular file extension

	webpack_isomorphic_tools.prototype.register_extension = function register_extension(extension, description) {
		var _this3 = this;

		this.log.debug(' registering a require() hook for *.' + extension);

		// place the require() hook for this extension
		this.hooks.push(_requireHacker2['default'].hook(extension, function (path) {
			return _this3.require(path, description);
		}));
	};

	// require()s an asset by a path

	webpack_isomorphic_tools.prototype.require = function require(global_asset_path, description) {
		this.log.debug('require() called for ' + global_asset_path);

		// sanity check
		/* istanbul ignore if */
		if (!this.options.project_path) {
			throw new Error('You forgot to call the .server() method passing it your project\'s base path');
		}

		// // if the require()d file is not part of the project - skip it
		// // (should be faster, but who needs faster if the modules are cached by Node.js)
		// if (!global_asset_path.indexOf(this.options.project_path) === 0)
		// {
		// 	this.log.debug(` skipping require call for ${asset_path} (not in project folder)`)
		// 	return
		// }

		// // asset path relative to the project folder
		// // (should be faster, but who needs faster if the modules are cached by Node.js)
		// let asset_path = global_asset_path.slice(this.options.project_path.length + fs.sep.length)

		// asset path relative to the project folder
		var asset_path = _path3['default'].relative(this.options.project_path, global_asset_path);

		// for Windows:
		//
		// convert Node.js path to a correct Webpack path
		asset_path = asset_path.replace(/\\/g, '/');
		// add './' in the beginning if it's missing (for example, in case of Windows)
		if (asset_path.indexOf('.') !== 0) {
			asset_path = './' + asset_path;
		}

		// if this filename is in the user specified exceptions list
		// (or is not in the user explicitly specified inclusion list)
		// then fall back to the normal require() behaviour
		if (!this.includes(asset_path, description) || this.excludes(asset_path, description)) {
			this.log.debug(' skipping require call for ' + asset_path);
			return;
		}

		// track cached assets (only in development mode)
		if (this.options.development) {
			// mark this asset as cached
			this.cached_assets.push(global_asset_path);
		}

		// webpack has a shortcut from "node_modules"
		if (asset_path.indexOf('./node_modules/') === 0) {
			asset_path = asset_path.replace('./node_modules/', './~/');
		}

		// return CommonJS module source for this asset
		return _requireHacker2['default'].to_javascript_module_source(this.asset_source(asset_path));
	};

	// returns asset source by path (looks it up in webpack-assets.json)

	webpack_isomorphic_tools.prototype.asset_source = function asset_source(asset_path) {
		this.log.debug(' requiring ' + asset_path);

		// sanity check
		/* istanbul ignore if */
		if (!asset_path) {
			return undefined;
		}

		// get real file path list
		var assets = this.assets().assets;

		// find this asset in the real file path list
		var asset = assets[asset_path];

		// if the asset was found in the list - return it
		if (_helpers.exists(asset)) {
			return asset;
		}

		// serve a not-found asset maybe
		this.log.error('asset not found: ' + asset_path);
		return undefined;
	};

	// unregisters require() hooks

	webpack_isomorphic_tools.prototype.undo = function undo() {
		// for each user specified asset type,
		// unregister a require() hook for each file extension of this asset type
		for (var _iterator4 = this.hooks, _isArray4 = Array.isArray(_iterator4), _i4 = 0, _iterator4 = _isArray4 ? _iterator4 : _getIterator(_iterator4);;) {
			var _ref4;

			if (_isArray4) {
				if (_i4 >= _iterator4.length) break;
				_ref4 = _iterator4[_i4++];
			} else {
				_i4 = _iterator4.next();
				if (_i4.done) break;
				_ref4 = _i4.value;
			}

			var hook = _ref4;

			hook.unmount();
		}

		// unmount the aliasing hook (if mounted)
		if (this.alias_hook) {
			this.alias_hook.unmount();
		}
	};

	// Checks if the required path should be excluded from the custom require() hook

	webpack_isomorphic_tools.prototype.excludes = function excludes(path, options) {
		// if "exclude" parameter isn't specified, then exclude nothing
		if (!_helpers.exists(options.exclude)) {
			return false;
		}

		// for each exclusion case
		for (var _iterator5 = options.exclude, _isArray5 = Array.isArray(_iterator5), _i5 = 0, _iterator5 = _isArray5 ? _iterator5 : _getIterator(_iterator5);;) {
			var _ref5;

			if (_isArray5) {
				if (_i5 >= _iterator5.length) break;
				_ref5 = _iterator5[_i5++];
			} else {
				_i5 = _iterator5.next();
				if (_i5.done) break;
				_ref5 = _i5.value;
			}

			var exclude = _ref5;

			// supports regular expressions
			if (exclude instanceof RegExp) {
				if (exclude.test(path)) {
					return true;
				}
			}
			// check for a compex logic match
			else if (typeof exclude === 'function') {
				if (exclude(path)) {
					return true;
				}
			}
			// otherwise check for a simple textual match
			else {
				if (exclude === path) {
					return true;
				}
			}
		}

		// no matches found.
		// returns false so that it isn't undefined (for testing purpose)
		return false;
	};

	// Checks if the required path should be included in the custom require() hook

	webpack_isomorphic_tools.prototype.includes = function includes(path, options) {
		// if "include" parameter isn't specified, then include everything
		if (!_helpers.exists(options.include)) {
			return true;
		}

		// for each inclusion case
		for (var _iterator6 = options.include, _isArray6 = Array.isArray(_iterator6), _i6 = 0, _iterator6 = _isArray6 ? _iterator6 : _getIterator(_iterator6);;) {
			var _ref6;

			if (_isArray6) {
				if (_i6 >= _iterator6.length) break;
				_ref6 = _iterator6[_i6++];
			} else {
				_i6 = _iterator6.next();
				if (_i6.done) break;
				_ref6 = _i6.value;
			}

			var include = _ref6;

			// supports regular expressions
			if (include instanceof RegExp) {
				if (include.test(path)) {
					return true;
				}
			}
			// check for a compex logic match
			else if (typeof include === 'function') {
				if (include(path)) {
					return true;
				}
			}
			// otherwise check for a simple textual match
			else {
				if (include === path) {
					return true;
				}
			}
		}

		// no matches found.
		// returns false so that it isn't undefined (for testing purpose)
		return false;
	};

	// Waits for webpack-assets.json to be created after Webpack build process finishes
	//
	// The callback is called when `webpack-assets.json` has been found
	// (it's needed for development because `webpack-dev-server`
	//  and your application server are usually run in parallel).
	//

	webpack_isomorphic_tools.prototype.wait_for_assets = function wait_for_assets(done) {
		var _this4 = this;

		// condition check interval
		var interval = 300; // in milliseconds

		// selfie
		var tools = this;

		// waits for condition to be met, then proceeds
		function wait_for(condition, proceed) {
			function check() {
				// if the condition is met, then proceed
				if (condition()) {
					return proceed();
				}

				tools.log.debug('(' + tools.webpack_assets_path + ' not found)');
				tools.log.info('(waiting for the first Webpack build to finish)');

				setTimeout(check, interval);
			}

			check();
		}

		// wait for webpack-assets.json to be written to disk by Webpack
		wait_for(function () {
			return _fs2['default'].existsSync(_this4.webpack_assets_path);
		}, done);

		// allows method chaining
		return this;
	};

	return webpack_isomorphic_tools;
})();

exports['default'] = webpack_isomorphic_tools;
module.exports = exports['default'];

// require() hooks for assets

// used to keep track of cached assets and flush their caches on .refresh() call
//# sourceMappingURL=index.js.map