'use strict';

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;
exports['default'] = write_assets;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _requireHacker = require('require-hacker');

var _requireHacker2 = _interopRequireDefault(_requireHacker);

var _toolsSerializeJavascript = require('../tools/serialize-javascript');

var _toolsSerializeJavascript2 = _interopRequireDefault(_toolsSerializeJavascript);

var _helpers = require('../helpers');

var _common = require('../common');

// writes webpack-assets.json file, which contains assets' file paths

function write_assets(json, options, log) {
	// take the passed in options
	options = _helpers.clone(options);

	// make webpack stats accessible for asset functions (parser, naming, filter)
	options.webpack_stats = json;

	log.debug('running write assets webpack plugin');

	var development = options.development;

	if (development) {
		log.debug(' (development mode is on)');
	}

	// write webpack stats json for debugging purpose
	if (options.debug) {
		// path to webpack stats file
		var webpack_stats_file_path = get_webpack_stats_file_path(options.webpack_assets_path);

		// write webpack stats file
		log.debug('writing webpack stats to ' + webpack_stats_file_path);

		// write the file
		// (format the JSON for better readability)
		_fsExtra2['default'].outputFileSync(webpack_stats_file_path, JSON.stringify(json, null, 2));
	}

	// the output object with assets
	var output = options.output;

	// populate the output object with assets
	populate_assets(output, json, options, log);

	// write webpack assets info file
	log.debug('writing webpack assets info to ' + options.webpack_assets_path);
	// format the JSON for better readability if in debug mode
	var assets_info = development ? JSON.stringify(output, null, 2) : JSON.stringify(output);
	// write the file
	_fsExtra2['default'].outputFileSync(options.webpack_assets_path, assets_info);
}

// populates the output object with assets
function populate_assets(output, json, options, log) {
	// for each chunk name ("main", "common", ...)
	_Object$keys(json.assetsByChunkName).forEach(function (name) {
		log.debug('getting javascript and styles for chunk "' + name + '"');

		// get javascript chunk real file path

		var javascript = get_assets(name, 'js')[0];
		// the second asset is usually a source map

		if (javascript) {
			log.debug(' (got javascript)');
			output.javascript[name] = javascript;
		}

		// get style chunk real file path

		var style = get_assets(name, 'css')[0];
		// the second asset is usually a source map

		if (style) {
			log.debug(' (got style)');
			output.styles[name] = style;
		}
	});

	// gets asset paths by name and extension of their chunk
	function get_assets(name) {
		var extension = arguments.length <= 1 || arguments[1] === undefined ? 'js' : arguments[1];

		var chunk = json.assetsByChunkName[name];

		// a chunk could be a string or an array, so make sure it is an array
		if (!Array.isArray(chunk)) {
			chunk = [chunk];
		}

		return chunk
		// filter by extension
		.filter(function (name) {
			return _path2['default'].extname(name) === '.' + extension;
		})
		// adjust the real path (can be http, filesystem)
		.map(function (name) {
			return options.assets_base_url + name;
		});
	}

	// // output assets for all application javascript entry points
	// Object.keys(this.options.entry).forEach(chunk_name =>
	// {
	// 	let entry = this.options.entry[chunk_name]
	// 	if (Array.isArray(entry))
	// 	{
	// 		entry = entry[entry.length - 1]
	// 	}
	//
	// 	output.javascript[entry] = get_assets(chunk_name, 'js')[0]
	// 	// the second asset is usually a source map
	// })

	// // omit node_modules contents and internal webpack modules
	// const modules = json.modules.filter(module =>
	// {
	// 	return module.name.indexOf('.') === 0 && module.name.indexOf('./~/') !== 0
	// 	// return module.name.indexOf('./~/') !== 0 && module.name.indexOf('(webpack)') !== 0
	// })

	// one can supply a custom filter
	var default_filter = function default_filter(module, regular_expression) {
		return regular_expression.test(module.name);
	};
	// one can supply a custom namer
	var default_asset_path = function default_asset_path(module) {
		return module.name;
	};
	// one can supply a custom parser
	var default_parser = function default_parser(module) {
		return module.source;
	};

	// 1st pass
	var parsed_assets = {};

	// global paths to parsed asset paths
	var global_paths_to_parsed_asset_paths = {};

	// define __webpack_public_path__ webpack variable
	// (resolves "ReferenceError: __webpack_public_path__ is not defined")
	var define_webpack_public_path = 'var __webpack_public_path__ = ' + JSON.stringify(options.assets_base_url) + ';\n';

	var _loop = function () {
		if (_isArray) {
			if (_i >= _iterator.length) return 'break';
			_ref = _iterator[_i++];
		} else {
			_i = _iterator.next();
			if (_i.done) return 'break';
			_ref = _i.value;
		}

		var asset_type = _ref;

		// asset type settings
		var asset_type_settings = options.assets[asset_type];

		// one can supply his own filter
		var filter = asset_type_settings.filter || default_filter; //.bind(this)
		// one can supply his own path parser
		var extract_asset_path = asset_type_settings.path || default_asset_path; //.bind(this)
		// one can supply his own parser
		var parser = asset_type_settings.parser || default_parser; //.bind(this)

		// guard agains typos, etc

		// for filter
		if (!asset_type_settings.filter) {
			log.debug('No filter specified for "' + asset_type + '" assets. Using a default one.');
		}

		// for path parser
		if (!asset_type_settings.path) {
			log.debug('No path parser specified for "' + asset_type + '" assets. Using a default one.');
		}

		// for parser
		if (!asset_type_settings.parser) {
			log.debug('No parser specified for "' + asset_type + '" assets. Using a default one.');
		}

		log.debug('parsing assets of type "' + asset_type + '"');

		// timer start
		var began_at = new Date().getTime();

		// get real paths for all the files from this asset type
		json.modules
		// take just modules of this asset type
		.filter(function (module) {
			// check that this asset is of the asset type
			if (!filter(module, options.regular_expressions[asset_type], options, log)) {
				return false;
			}

			// guard against an empty source.
			if (!module.source) {
				log.error('Module "' + module.name + '" has no source. Maybe Webpack compilation of this module failed. Skipping this asset.');
				return false;
			}

			// include this asset
			return true;
		}).reduce(function (set, module) {
			// determine asset real path
			var asset_path = extract_asset_path(module, options, log);

			// (no need to resolve require() paths because it just works as is)
			//
			// const resolve_require_paths = (module_source) =>
			// {
			// 	return module_source.replace(/(\s)require\("([^"]+)"\)/g, function(match, space_before, required_path)
			// 	{
			// 		const asset_folder = path.dirname(asset_path)
			// 		const required_file_path = path.join(asset_folder, required_path)
			//
			// 		const required_file_absolute_path = path.resolve(options.project_folder, required_file_path)
			// 		const node_modules_path = path.join(options.project_folder, 'node_modules')
			//
			// 		// if it's a require() call for a file inside "node_modules",
			// 		// then adjust the required path accordingly
			// 		if (required_file_absolute_path.indexOf(node_modules_path) === 0)
			// 		{
			// 			// account for the last '/'
			// 			required_path = required_path.slice(node_modules_path.length + 1)
			// 		}
			// 		else
			// 		{
			// 			required_path = './' + replace_all(required_file_path, path.sep, '/')
			// 		}
			//
			// 		return space_before + `require("${required_path}")`
			// 	})
			// }

			// asset module source, or asset content (or whatever else)
			var parsed_asset = parser(module, options, log);

			log.trace('Adding assset "' + asset_path + '", module id ' + module.id + ' (in webpack-stats.debug.json)');

			// check for naming collisions (just in case)
			if (_helpers.exists(set[asset_path])) {
				log.error('-----------------------------------------------------------------');
				log.error('Asset with path "' + asset_path + '" was overwritten because of path collision.');
				log.error('Use the "filter" function of this asset type to narrow the results.');
				log.error('Previous asset with this path:');
				log.error(set[asset_path]);
				log.error('New asset with this path:');
				log.error(parsed_asset);
				log.error('-----------------------------------------------------------------');
			}

			// add this asset to the list
			//
			// also resolve "ReferenceError: __webpack_public_path__ is not defined".
			// because it may be a url-loaded resource (e.g. a font inside a style).
			set[asset_path] = define_webpack_public_path + _requireHacker2['default'].to_javascript_module_source(parsed_asset);

			// add path mapping
			global_paths_to_parsed_asset_paths[_path2['default'].resolve(options.project_path, asset_path)] = asset_path;

			// continue
			return set;
		}, parsed_assets);

		// timer stop
		log.debug(' time taken: ' + (new Date().getTime() - began_at) + ' ms');
	};

	// for each user specified asset type
	for (var _iterator = _Object$keys(options.assets), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _getIterator(_iterator);;) {
		var _ref;

		var _ret = _loop();

		if (_ret === 'break') break;
	}

	// register a special require() hook for requiring() raw webpack modules
	var require_hook = _requireHacker2['default'].global_hook('webpack-module', function (required_path) {
		// if Webpack aliases are supplied
		if (options.alias) {
			// alias the path
			var aliased_path = _common.alias(required_path, options.alias);

			// if an alias is found, require() the correct path
			if (aliased_path) {
				log.debug('alias found for ' + required_path + ', resolving to ' + aliased_path);

				var global_path = require('module')._resolveFilename(aliased_path, module);
				log.debug(' global path for aliased module is ' + global_path);

				var result = require(global_path); // require(aliased_path)
				return _requireHacker2['default'].to_javascript_module_source(result);
			}
		}

		// find an asset with this path
		if (_helpers.exists(global_paths_to_parsed_asset_paths[required_path])) {
			return parsed_assets[global_paths_to_parsed_asset_paths[required_path]];
		}

		// find a webpack module which has a reason with this path
		for (var _iterator2 = json.modules, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _getIterator(_iterator2);;) {
			var _ref2;

			if (_isArray2) {
				if (_i2 >= _iterator2.length) break;
				_ref2 = _iterator2[_i2++];
			} else {
				_i2 = _iterator2.next();
				if (_i2.done) break;
				_ref2 = _i2.value;
			}

			var _module2 = _ref2;

			for (var _iterator3 = _module2.reasons, _isArray3 = Array.isArray(_iterator3), _i3 = 0, _iterator3 = _isArray3 ? _iterator3 : _getIterator(_iterator3);;) {
				var _ref3;

				if (_isArray3) {
					if (_i3 >= _iterator3.length) break;
					_ref3 = _iterator3[_i3++];
				} else {
					_i3 = _iterator3.next();
					if (_i3.done) break;
					_ref3 = _i3.value;
				}

				var reason = _ref3;

				if (reason.userRequest === required_path) {
					// also resolve "ReferenceError: __webpack_public_path__ is not defined".
					// because it may be a url-loaded resource (e.g. a font inside a style).
					return define_webpack_public_path + _module2.source;
				}
			}
		}
	}, { precede_node_loader: true });

	log.debug('compiling assets');

	// timer start
	var began_at = new Date().getTime();

	// evaluate parsed assets source code
	for (var _iterator4 = _Object$keys(parsed_assets), _isArray4 = Array.isArray(_iterator4), _i4 = 0, _iterator4 = _isArray4 ? _iterator4 : _getIterator(_iterator4);;) {
		var _ref4;

		if (_isArray4) {
			if (_i4 >= _iterator4.length) break;
			_ref4 = _iterator4[_i4++];
		} else {
			_i4 = _iterator4.next();
			if (_i4.done) break;
			_ref4 = _i4.value;
		}

		var asset_path = _ref4;

		// set asset value
		output.assets[asset_path] = require(_path2['default'].resolve(options.project_path, asset_path));
	}

	// unmount the previously installed require() hook
	require_hook.unmount();

	// timer stop
	log.debug(' time taken: ' + (new Date().getTime() - began_at) + ' ms');
}

function get_webpack_stats_file_path(webpack_assets_file_path) {
	// default webpack stats file name
	var webpack_stats_file_name = 'webpack-stats.json';

	// resolve a possible file name collision
	/* istanbul ignore if */
	if (_path2['default'].basename(webpack_assets_file_path) === webpack_stats_file_name) {
		webpack_stats_file_name = 'webpack-stats.debug.json';
	}

	// path to webpack stats file
	return _path2['default'].resolve(_path2['default'].dirname(webpack_assets_file_path), webpack_stats_file_name);
}
module.exports = exports['default'];
//# sourceMappingURL=write assets.js.map